//
//  ViewController.m
//  locallocationnotify
//
//  Created by iPhone Developer on 2013/11/14.
//  Copyright (c) 2013年 iPhone Developer. All rights reserved.
//

#import <MapKit/MapKit.h>
#import "ViewController.h"
#import "MapAnnotation.h"

@interface ViewController ()
{
    CLLocationManager *locationManager_;
	CLLocation *startLocation_;
    float prevDistance_;
}

@property (strong, nonatomic) IBOutlet MKMapView *mapView;

@end

@implementation ViewController

@synthesize mapView = _mapView;
@synthesize runBackground = _runBackground;

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    // 初期位置は未設定
    startLocation_ = nil;
    // 初期位置と現在位置の距離は0m
    prevDistance_ = 0.0f;
	locationManager_ = [[CLLocationManager alloc] init];
	[locationManager_ setDelegate:self];
    // 位置情報の取得頻度を設定する(10m移動したら通知)
    [locationManager_ setDistanceFilter:(CLLocationDistance)10];
    // 位置情報の更新を開始する
	[locationManager_ startUpdatingLocation];
    // マップ上に現在地を示すマーカーを表示する
    [_mapView setShowsUserLocation:YES];
    // バックグラウンドで動作しているか？
    _runBackground = NO;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc
{
    // 位置情報の更新を停止する
    [locationManager_ stopUpdatingLocation];
    locationManager_ = nil;
}

#pragma mark -
#pragma mark CLLocationManager delegate

// ベッセル
#define BESSEL_A		6377397.155
#define BESSEL_E2		0.00667436061028297
#define BESSEL_MNUM	6334832.10663254

// 世界測地系(GRS80)
#define GRS80_A		6378137.000
#define GRS80_E2		0.00669438002301188
#define GRS80_MNUM	6335439.32708317

// 世界測地系(WGS84)
#define WGS84_A		6378137.000
#define WGS84_E2		0.00669437999019758
#define WGS84_MNUM	6335439.32729246

enum {
    BESSEL = 0,
    GRS80,
    WGS84
};

double deg2rad(double deg) {
    return deg * M_PI / 180.0;
}

double calcDistHubenyWithGeom(double lat1, double lng1,
                              double lat2, double lng2,
                              double a, double e2, double mnum) {
    double my = deg2rad((lat1 + lat2) / 2.0);
    double dy = deg2rad(lat1 - lat2);
    double dx = deg2rad(lng1 - lng2);
    
    double sinv = sin(my);
    double w = sqrt(1.0 - e2 * sinv * sinv);
    double m = mnum / (w * w * w);
    double n = a / w;
    
    double dym = dy * m;
    double dxncos = dx * n * cos(my);
    
    return sqrt(dym * dym + dxncos * dxncos);
}

// 緯度経度で指定された2点間の距離を計算する
double calcDistHubenyWithoutGeom(double lat1, double lng1,
                                 double lat2, double lng2){
    return calcDistHubenyWithGeom(lat1, lng1, lat2, lng2,
                                  GRS80_A, GRS80_E2, GRS80_MNUM);
}

// 緯度経度で指定された2点間の距離を計算する(測地系を指定)
double calcDistHubery(double lat1, double lng1,
                      double lat2, double lng2, int type){
    switch ( type ) {
        case BESSEL:
            return calcDistHubenyWithGeom(lat1, lng1, lat2, lng2,
                                          BESSEL_A, BESSEL_E2, BESSEL_MNUM);
        case WGS84:
            return calcDistHubenyWithGeom(lat1, lng1, lat2, lng2,
                                          WGS84_A, WGS84_E2, WGS84_MNUM);
        default:
            return calcDistHubenyWithGeom(lat1, lng1, lat2, lng2,
                                          GRS80_A, GRS80_E2, GRS80_MNUM);
    }
}

// Locationのデータがアップデートした際に呼ばれる
- (void)locationManager:(CLLocationManager *)manager
    didUpdateToLocation:(CLLocation *)newLocation
           fromLocation:(CLLocation *)oldLocation
{
    NSString *locationString = [NSString stringWithFormat:@"%lf,%lf", newLocation.coordinate.latitude, newLocation.coordinate.longitude];
    NSLog(@"locationString: %@", locationString);
	if ( !startLocation_ ) {
        // 初期位置が記録されていなければ初期の位置情報を記録する
		startLocation_ = [[CLLocation alloc] initWithLatitude:newLocation.coordinate.latitude longitude:newLocation.coordinate.longitude];
        // マップ上の初期位置にピンを立てる
        MapAnnotation *annotation = [[MapAnnotation alloc] initWithCoordinate:startLocation_.coordinate withTitle:@"スタート" withSubtitle:[NSString stringWithFormat:@"%f, %f", startLocation_.coordinate.latitude, startLocation_.coordinate.latitude]];
        NSArray *annotations = [NSArray arrayWithObjects:annotation, nil];
        // mapView にannotationを登録
        [_mapView addAnnotations:annotations];
	}
	else {
        // 既に初期位置が記録されていたならば、初期位置と現在位置の距離を計算する
		float distance = calcDistHubenyWithoutGeom(startLocation_.coordinate.latitude, startLocation_.coordinate.longitude, newLocation.coordinate.latitude, newLocation.coordinate.longitude);
        if ( !_runBackground && distance != prevDistance_ ) {
            // 計算した距離が前回の計算と違っていたならば通知センターで位置情報を通知する
            // UILocalNotificationのクラスインスタンスを生成
            UILocalNotification *localNotification = [[UILocalNotification alloc] init];
            // アラートには初期位置からの移動距離を設定する
            localNotification.alertBody = [NSString stringWithFormat:@"初期位置から %f km移動しました", distance / 1000];
            // アクションボタンあり
            localNotification.hasAction = YES;
            // アクションボタンには「アプリを起動」を表示する
            localNotification.alertAction = @"アプリを起動";
            // 通知を受信したらデフォルトの通知音を鳴らす
            localNotification.soundName = UILocalNotificationDefaultSoundName;
            // アプリケーション独自の情報としてディクショナリを送信(受信時の処理は自分で実装しなければならない)
            NSDictionary *userInfo = [NSDictionary dictionaryWithObject:@"通知を受信しました。" forKey:@"ExtraInfo"];
            localNotification.userInfo = userInfo;
            // Local Notificationをスケジュール(presentLocalNotificationNowなので即時に通知される)
            [[UIApplication sharedApplication] presentLocalNotificationNow:localNotification];
            localNotification = nil;
        }
        prevDistance_ = distance;
    }
    
    // 地図の中心点をGPSから取得された位置に移動
    CLLocationCoordinate2D center = newLocation.coordinate;
    [_mapView setCenterCoordinate:center animated:YES];
}

// Locationのデータの取得に失敗した際に呼ばれる
- (void)locationManager:(CLLocationManager *)manager
       didFailWithError:(NSError *)error
{
    NSLog(@"GPS情報の取得に失敗しました。");
    /* アラートを表示するなら
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"GPS Error" message:@"Error" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    alert = nil;
    */
}

@end
